using System;

using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.Ecommerce;

/// <summary>
/// WorldPayLoader e-commerce loader class. Partial class ensures correct registration.
/// </summary>
/// 
[WorldPayLoader]
public partial class CMSModuleLoader
{
    #region "Macro methods loader attribute"

    /// <summary>
    /// Module registration
    /// </summary>
    private class WorldPayLoaderAttribute : CMSLoaderAttribute
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public WorldPayLoaderAttribute()
        {
            // Require E-commerce module to load properly
            RequiredModules = new string[] { ModuleEntry.ECOMMERCE };
        }


        /// <summary>
        /// Initializes the module
        /// </summary>
        public override void Init()
        {
            // This line provides the ability to register the classes via web.config cms.extensibility section from App_Code
            ClassHelper.OnGetCustomClass += GetCustomClass;
        }


        /// <summary>
        /// Gets the custom class object based on the given class name. This handler is called when the assembly name is App_Code.
        /// </summary>
        private static void GetCustomClass(object sender, ClassEventArgs e)
        {
            if (e.Object == null)
            {
                // Provide your custom classes
                switch (e.ClassName.ToLower())
                {
                    // Define the class WorldPayProvider inheriting the CMSPaymentGatewayProvider and you can customize the provider
                    case "worldpayprovider":
                        e.Object = new WorldPayProvider();
                        break;
                }
            }
        }
    }

    #endregion
}
