﻿using System;
using System.Collections.Generic;
using System.Text;

using CMS.GlobalHelper;

namespace CMS.Ecommerce
{
    /// <summary>
    /// WorldPay payment result.
    /// </summary>
    public class WorldPayPaymentResultInfo : PaymentResultInfo
    {
        private const string HEADER_VERIFIED = "{$PaymentGateway.WorldPayResult.Verification$}";

        /// <summary>
        /// WorldPay payment status.
        /// </summary>
        public WorldPayPaymentStatusEnum WorldPayPaymentStatus
        {
            get
            {
                return GetWorldPayPaymentStatusEnum();
            }
            set
            {
                SetWorldPayPaymentStatus(value);
            }
        }


        /// <summary>
        /// TRUE - payment was verified by PN (Payment notification) successfully, payment is VALID.
        /// FALSE - payment verification by PN failed, payment is INVALID.
        /// </summary>
        public bool WorldPayPaymentVerified
        {
            get
            {
                PaymentResultItemInfo itemObj = this.EnsurePaymentResultItemInfo("verified", HEADER_VERIFIED);
                if (itemObj != null)
                {
                    return ValidationHelper.GetBoolean(itemObj.Value, false);
                }
                return false;
            }
            set
            {
                PaymentResultItemInfo item = EnsurePaymentResultItemInfo("verified", HEADER_VERIFIED);
                if (item != null)
                {
                    if (ValidationHelper.GetBoolean(value, false))
                    {
                        item.Value = "1";
                        item.Text = "{$PaymentGateway.WorldPayResult.Verification.Verified$}";
                    }
                    else
                    {
                        item.Value = "0";
                        item.Text = "{$PaymentGateway.WorldPayResult.Verification.Invalid$}";
                    }

                    this.SetPaymentResultItemInfo(item);
                }
            }
        }


        /// <summary>
        /// Returns WorldPay payment status according to the returned status string from the WorldPay payment gateway.
        /// </summary>        
        /// <param name="status">Payment status string returned from the WorldPay payment gateway</param>
        public static WorldPayPaymentStatusEnum GetWorldPayPaymentStatus(string status)
        {
            switch (status.ToLower())
            {
                case "y":
                    return WorldPayPaymentStatusEnum.Completed;

                case "c":
                    return WorldPayPaymentStatusEnum.Failed;

                default:
                    return WorldPayPaymentStatusEnum.Unknown;
            }
        }


        #region " Private methods"

        /// <summary>
        /// Sets WorldPay payment status according to the returned status from tha WorldPay payment gateway.
        /// </summary>        
        /// <param name="status">Payment status string returned from the WorldPay payment gateway</param>
        private void SetWorldPayPaymentStatus(WorldPayPaymentStatusEnum status)
        {
            switch (status)
            {
                case WorldPayPaymentStatusEnum.Completed:
                    this.PaymentStatusValue = "y";
                    this.PaymentStatusName = "{$WorldPay.PaymentResult.Status.Completed$}";
                    break;

                case WorldPayPaymentStatusEnum.Failed:
                    this.PaymentStatusValue = "c";
                    this.PaymentStatusName = "{$WorldPay.PaymentResult.Status.Failed$}";
                    break;

                default:
                    this.PaymentStatusValue = "";
                    this.PaymentStatusName = "{$WorldPay.PaymentResult.Status.Unknown$}";
                    break;
            }
        }


        /// <summary>
        /// Returns WorldPay payment status enumeration from payment result status string.
        /// </summary>        
        private WorldPayPaymentStatusEnum GetWorldPayPaymentStatusEnum()
        {
            switch (this.PaymentStatusValue.ToLower())
            {
                case "y":
                    return WorldPayPaymentStatusEnum.Completed;

                case "c":
                    return WorldPayPaymentStatusEnum.Failed;

                default:
                    return WorldPayPaymentStatusEnum.Unknown;
            }
        }

        #endregion


        /// <summary>
        /// Creates base payment result info object + add WorldPay payment result properties.
        /// </summary>
        public WorldPayPaymentResultInfo()
            : base()
        {
            // Add payment verification - not known
            EnsurePaymentResultItemInfo("verified", HEADER_VERIFIED);
        }
    }
}
